// ---------------------------------------------------------------------------
// jump.cs -- Version 2.7 -- October 28, 2000
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// Hold or tap a key to keep jump, jump, jumping -- the skier's best friend. :)
//
// Note: if another script needs the events generated by jump.cs but you don't
//       want to change how the jump key works, just set $Jump::interval to 0.
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\jump.cs", "2.7", "Lorne Laliberte", "Oct 28, 2000");

include("writer\\event.cs");
include("writer\\save.cs");
include("writer\\tap.cs");
include("writer\\whatif.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_jump.cs file

    // Enable "continuous jumping when the key is tapped" mode by default
    $Jump::KeyTapEnabled = "false";

    // Tap time in seconds -- within which the jump key (or button) must be
    // pressed and released to be recognized as a key tap
    $Jump::tapTime = "0.100";

    // Jump interval in seconds -- delay between each jump when you hold the
    // jump key down (or after you tap the jump key with the key tap enabled)
    // (Set this to 0 if you want to disable the continous jumping feature)
    $Jump::interval = "0.01";

    // Set this to your jump key (or button)
    //bind(keyboard0, make, "space", to, "JumpPressed();");
    //bind(keyboard0, break, "space", to, "JumpReleased();");

    // Toggle the "continuous jumping when the key is tapped" mode on/off
    //bind(keyboard0, make, control, "space", to, "Jump::ToggleKeyTap();");
    //bind(keyboard0, break, control, "space", to, "", "Jump::ToggleKeyTap_break");

// ---------------------------------------------------------------------------

WriterPrefs::load("jump");

if(!$Jump::tapTime) $Jump::taptime = "0.100";

function JumpPressed(%taptime)
{
    if(!%taptime)
        %taptime = $Jump::tapTime;

    Event::Trigger(eventJumpPressed);

    if( !Event::Returned(eventJumpPressed, mute) )
    {
        if(!$Jump::interval || (%taptime == -1))
        {
            Event::Trigger(eventJump);
            if(Event::Returned(eventJump, mute))
                return;

        	postAction(2048, IDACTION_MOVEUP, -0); // Jump!
            return;
        }

        $Jumping = 1;
        KeepJumping();
        tap(jump, %taptime);
    }
}


function JumpReleased()
{
    Event::Trigger(eventJumpReleased);

    if( !Event::Returned(eventJumpReleased, mute) )
        $Jumping = 0;
}


function onJumpPressed()
{
    if($Jumping)
    {
        $Jumping = 0;
        return mute;
    }
    return;
}


function onJumpReleased()
{
    if(wasTapped(Jump))
        return mute;

    $Jumping = 0;
    return;
}


// Prevent wild jumping in observer mode
function Jump::onJump()
{
    if(Client::Getteam(getmanagerid()) == -1)
    {
    	postAction(2048, IDACTION_MOVEUP, -0); // Jump!
        return mute;
    }
    return;
}


// Keep jumping while a key is held down
//
function KeepJumping()
{
    // This function will keep calling itself so we need to set or clear
    // a control flag in a separate function -- if you want this to sound
    // technical, "we need external state control." :)

    if($Jumping)
    {
        // This event is going to be called very often, so try to keep the attached
        // functions fast and efficient!

        Event::Trigger(eventJump);

        // Note that returning mute stops the jumping even if the jump key is held down!

        if(Event::Returned(eventJump, mute))
            return;

    	postAction(2048, IDACTION_MOVEUP, -0); // Jump!

    	// Call this function in $Jump::interval seconds to jump again
    	// so long as $Jumping hasn't been cleared
    	schedule("KeepJumping();", $Jump::interval); // interval has to be small to always land with a jump
    }
}


// Toggle the "continuous jumping when key is tapped" mode on/off
//
function Jump::ToggleKeyTap()
{
    remoteBP(2048, "<L5>Tap Jump Mode <f2>" @ wif($Jump::KeyTapEnabled = !$Jump::KeyTapEnabled, "ON", "OFF"), 4);

    if($Jump::KeyTapEnabled)
    {
        Event::Attach(eventJumpPressed, "onJumpPressed();");
        Event::Attach(eventJumpReleased, "onJumpReleased();");
    }
    else
    {
        Event::Detach(eventJumpPressed, "onJumpPressed();");
        Event::Detach(eventJumpReleased, "onJumpReleased();");
    }
}

function jump::save()
{
    AddExportText("    // Enable \"continuous jumping when the key is tapped\" mode by default");
    ExportSetting("    $Jump::KeyTapEnabled", false);
    AddExportText("");
    AddExportText("    // Tap time in seconds -- within which the jump key (or button) must be");
    AddExportText("    // pressed and released to be recognized as a key tap");
    ExportSetting("    $Jump::tapTime", "0.100");
    AddExportText("");
    AddExportText("    // Jump interval in seconds -- delay between each jump when you hold the");
    AddExportText("    // jump key down (or after you tap the jump key with the key tap enabled)");
    AddExportText("    // (Set this to 0 if you want to disable the continous jumping feature)");
    ExportSetting("    $Jump::interval", "0.01");
    AddExportText("");
    AddExportText("    // Set this to your jump key (or button)");

    ExportBind("JumpPressed();", "//bind(keyboard0, make, \"space\", TO, \"JumpPressed();\");");
    ExportBind("JumpReleased();", "//bind(keyboard0, break, \"space\", TO, \"JumpReleased();\");");

    AddExportText("");
    AddExportText("    // Toggle the \"continuous jumping when the key is tapped\" mode on/off");

    ExportBind("Jump::ToggleKeyTap();", "//bind(keyboard0, make, control, \"space\", TO, \"Jump::ToggleKeyTap();\");");
    ExportBind("Jump::ToggleKeyTap_break", "//bind(keyboard0, break, control, \"space\", TO, \"\", Jump::ToggleKeyTap_break);");
}

WriterPrefs::save("jump");

function jump::init()
{
    needs("writer\\jump.cs", "writer\\event.cs 2.7",
                             "writer\\save.cs 2.3",
                             "writer\\tap.cs 1.2",
                             "writer\\whatif.cs 1.1");

    if($Jump::KeyTapEnabled)
    {
        Event::Attach(eventJumpPressed, "onJumpPressed();");
        Event::Attach(eventJumpReleased, "onJumpReleased();");
    }
    Event::Attach(eventJump, "Jump::onJump();");
}
Event::Attach(writerInit, "jump::init();");

if(!$UsingWriterLoader) jump::init();
